(function($) {
    $.gamesRegistry('register', 'zoom', new (function() {
	var options = {
	    'view_width': Infinity,
	    'view_height': Infinity,
	    'view_background_img': '',
	    'zoom_area': {
		'width': Infinity,
		'height': Infinity,
		'x': 0,
		'y': 0,
	    },
	    'zoom_zones': [],
	    glass: null
	};

	var glassPrototype = {
	    image: "",
	    width: Infinity,
	    height: Infinity,
	    transform_origin_x: "",
	    transform_origin_y: "",
	    transform_style: ""
	};

	this.playSound = function(name)
	{   
	    if (snd_instance) {
	    	snd_instance.stop();
	    }
	    
	    if (sound_is_off) {
	    	return;
	    }

	    snd_instance = createjs.Sound.play(name);

	};

	this.init = function(data) {
	    $.extend(options, data);
	    var game = this;

	    var background = preload.getResult(options.view_background_img).src;
	    $('#playground').css('background-image', 'url("' + background + '")')

	    if (options.zoom_area.background !== undefined) {
		var zoomAreaBg = $('<div class="zoom-area"></div>');
		zoomAreaBg.width(100 * options.zoom_area.width / options.view_width + '%');
		zoomAreaBg.height(100 * options.zoom_area.height / options.view_height + '%');
		zoomAreaBg.css({
		    left: 100 * options.zoom_area.x / options.view_width + '%',
		    top: 100 * options.zoom_area.y / options.view_height + '%',
		    'background-image': 'url("' + preload.getResult(options.zoom_area.background).src + '")'
		});

		$('#playground').append(zoomAreaBg);

	    }

	    var zoomArea = $('<div class="zoom-area animate"></div>');
	    zoomArea.width(100 * options.zoom_area.width / options.view_width + '%');
	    zoomArea.height(100 * options.zoom_area.height / options.view_height + '%');
	    zoomArea.css({
		left: 100 * options.zoom_area.x / options.view_width + '%',
		top: 100 * options.zoom_area.y / options.view_height + '%'
	    });
	    $('#playground').append(zoomArea);

	    $.each(options.objects, function() {
		var obj = $('<div class="object"></div>');
		obj.css('background-image', 'url("' + preload.getResult(this.image).src + '")');

		obj.width(100 * this.width / options.view_width + '%');
		obj.height(100 * this.height / options.view_height + '%');
		obj.css({
		    left: 100 * this.x / options.view_width + '%',
		    top: 100 * this.y / options.view_height + '%',
		});

		$('#playground').append(obj);
	    });

	    // Zoom glass
	    var zoomGlass;
	    if (options.glass != null) {
		var glassOptions = glassPrototype;
		$.extend(glassOptions, options.glass);
		$.extend(glassOptions, {x: 0, y: 0});
		zoomGlass = $('<div class="zoom-glass"></div>');

		area = new absoluteArea(options.glass);
		area.applyAt(zoomGlass, {
		    view_width: options.view_width,
		    view_height: options.view_height,
		});

		var background = preload.getResult(options.glass.image).src;
		zoomGlass.css('background-image', 'url("' + background + '")');
		zoomGlass.css('transform-origin', 100 * options.glass.transform_origin_x / options.glass.width + '%' + " " + 100 * options.glass.transform_origin_y / options.glass.height + '%');
		zoomGlass.data('options', options.glass);
		zoomGlass.css({
		    left: '-10000%',
		    top: '-10000%'
		});
		$('#playground').append(zoomGlass);
	    }

	    var text = $('<div id="text_panel" style="font-family: Andada; position: absolute; font-size:1.1em; color:#000; dispaly:block;"></div>');

	    var obj = {x: 150, y: 1100, width: 2400, height: 200};

	    area = new absoluteArea(obj);
	    area.applyAt(text, {
		view_width: options.view_width,
		view_height: options.view_height,
	    });

	    $('#playground').append(text);

	    // Create zoom zones
	    $.each(options.zoom_zones, function() {
		var zoomZone = $('<div class="zoom-zone circle"></div>');

		var self = this;

		area = new absoluteArea(this);
		area.applyAt(zoomZone, {
		    view_width: options.view_width,
		    view_height: options.view_height,
		});

		zoomZone.data('options', this);
		zoomZone.bind('click', function() {
		    var zoneOptions = $(this).data('options');
		    var glassOptions = zoomGlass.data('options');
		    var background = preload.getResult(zoneOptions.image).src;

		    $('#text_panel').html('');


		    if (self.snd !== undefined) {
			game.playSound(self.snd);
		    }

		    zoomArea.addClass('fade').on('transitionend webkitTransitionEnd oTransitionEnd otransitionend', function(e) {
			zoomArea.unbind('transitionend webkitTransitionEnd oTransitionEnd otransitionend');
			zoomArea.css('background-image', 'url("' + background + '")');
			zoomArea.removeClass('fade');
			if (self.text !== undefined)
			    $('#text_panel').html(self.text);
		    });

		    var px = zoneOptions.x + zoneOptions.width / 2 - glassOptions.transform_origin_x;
		    var py = zoneOptions.y + zoneOptions.height / 2 - glassOptions.transform_origin_y;

		    zoomGlass.css({
			left: 100 * px / options.view_width + '%',
			top: 100 * py / options.view_height + '%'
		    });

		    if (zoneOptions.title !== undefined && zoneOptions.title !== '') {
			zoomArea.attr('title', zoneOptions.title);
		    }


		});
		zoomZone.bind('mouseleave', function() {
		    // zoomArea.css('background-image', 'none')
		});
		$('#playground').append(zoomZone);
	    });
	}
    })());
})(jQuery);

